;																				
; TIMING nach AN26 DALLAS / MAXIM												
;																				
;           ___                          ___				  __________		
; RESET        |	                    |   |				 |					
;              |________________________|	|_____________|__|               	
;               <       480s           ><  70 s(SAMPLE)>|< 410s>            	
; 																				
;           ___      ____________________________________________________   	
; WRITE "1"    |    |															
;              |____|	                                						
;               <6s><         64 s              >             				
; 																				
;           ___                                    ______________________		
; WRITE "0"    |	                              |								
;              |__________________________________|	       						
;               <              60s              > 								
; 																				
;           ___      ______________|______________________________________   	
; READ         |    | |			   |                  |                         
;              |____| |____________|__________________|                         
;               <5s>< 9s(SAMPLE)>|<         55s           >	                
; 																				
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1-Wire, Reset und Presence-Test												
ONE_WIRE_RESET_PRESENCE:
	; Port auf Ausgang + LOW		
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	; RESET IMPULS 480s (500 s)	
	rcall ONE_WIRE_WAIT_500us
	; Port auf EINGANG + HIGH		
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	; Pause 70s (100s)			
	rcall ONE_WIRE_WAIT_100us
	cbi PORTD,LED_D_A						; LED-A	an		
	; SAMPLES						
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_RESET_PRESENCE_ERROR
	rjmp ONE_WIRE_RESET_PRESENCE_OK
;-------------------------------------------------------------------------------
ONE_WIRE_RESET_PRESENCE_OK:
	sbi PORTD,LED_D_A						; LED-A	aus		
	cbi PORTD,LED_D_GRUEN					; LED-GRN an	
	rcall ONE_WIRE_WAIT_500us				; PAUSE			
	cbi PORTD,LED_D_A						; LED-A	an		
	sbi PORTD,LED_D_A						; LED-A	aus		
	; vorsichtshalber den PIN auf AUSGANG (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_RESET_PRESENCE_ERROR:
	sbi PORTD,LED_D_A						; LED-A			
	cbi PORTD,LED_D_ROT						; LED-ROT   an	
	sbi PORTD,LED_D_GRUEN					; LED-GRN	aus	
	rcall ONE_WIRE_WAIT_500us				; PAUSE			
	cbi PORTD,LED_D_A						; LED-A			
	sbi PORTD,LED_D_A						; LED-A			
	; vorsichtshalber den PIN auf AUSGANG+HIGH (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1 DATENBYTE 1-wire schreiben													
;																				
; inp: temp1																	
;																				
; rcall DS_DATA_WRITE		; inp: temp1										
;																				
ONE_WIRE_DATA_WRITE:
	cbi PORTD,LED_D_B				; LED-B an		

	LSR temp1						; 1x rechts 1 rollt raus (LSB)
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-0 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-1 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-2 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-3 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-4 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-5 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus 	
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-6 schreiben			
	LSR temp1						; 1x rechts 1 rollt raus (MSB)
	rcall ONE_WIRE_DATA_WRITE_BIT	; BIT-7 schreiben			

	sbi PORTD,LED_D_B				; LED-B aus		

	; kurze Pause
	rcall ONE_WIRE_WAIT_500us	

ret
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_WRITE_BIT:
	; CARRY auswerten	
	brcs ONE_WIRE_DATA_WRITE_BIT_HIGH	; wenn CARRY=1 dann HIGH	
	rjmp ONE_WIRE_DATA_WRITE_BIT_LOW	; wenn CARRY=0 dann LOW 	
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_WRITE_BIT_LOW:
	; "0" SLOT 60...120s (LOW-PEGEL)							
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	rcall ONE_WIRE_WAIT_100us				; 100s PAUSE = "0" SLOT	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
ret
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_WRITE_BIT_HIGH:
	; "1" SLOT 1...15s LOW_PEGEL, dann mind. 45s HIGH-PEGEL	
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	rcall ONE_WIRE_WAIT_5us					; 5s PAUSE = "1" SLOT	
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	rcall ONE_WIRE_WAIT_100us				; 100s PAUSE = "1" SLOT	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 1 DATENBYTE 1-wire lesen														
;																				
; out: temp1																	
;																				
; rcall DS_DATA_READ		; OUT: temp1										
;																				
ONE_WIRE_DATA_READ:
	clr temp1

	rcall ONE_WIRE_DATA_READ_BIT	; LSB
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT
	rcall ONE_WIRE_DATA_READ_BIT	; MSB

	; vorsichtshalber den PIN auf AUSGANG+HIGH (zur Vermeidung von Streinflssen)
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	

	; kurze Pause
	rcall ONE_WIRE_WAIT_500us

	ret
;-------------------------------------------------------------------------------
; ein BIT vom SLAVE einlesen													
ONE_WIRE_DATA_READ_BIT:
	; kurzer LOW-IMPULS 1...5s
	sbi (ONE_WIRE_DDR) ,(ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	cbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; LOW-PEGEL		
	rcall ONE_WIRE_WAIT_5us					; 5s PAUSE		
	cbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF EINGANG	
	rcall ONE_WIRE_WAIT_10us				; ca 10s warten
	; MASTER SAMPLES			
	sbic (ONE_WIRE_PIN),(ONE_WIRE_PORTBIT) 	; "skip if..."
	rjmp ONE_WIRE_DATA_READ_1_SLOT
	rjmp ONE_WIRE_DATA_READ_0_SLOT
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_READ_0_SLOT:
	; LED		
	cbi PORTD,LED_D_C		; LED-C	an	
	sbi PORTD,LED_D_C		; LED-C	aus	
	; 1x rechts 
	LSR temp1
	; warten	
	rcall ONE_WIRE_WAIT_100us
	ret
;-------------------------------------------------------------------------------
ONE_WIRE_DATA_READ_1_SLOT:
	; LED		
	cbi PORTD,LED_D_C		; LED-C	an	
	sbi PORTD,LED_D_C		; LED-C	aus	
	; 1x rechts 
	LSR temp1
	; bit setzen
	ori temp1,0b10000000
	; warten	
	rcall ONE_WIRE_WAIT_100us
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 5s PAUSE																	
ONE_WIRE_WAIT_5us:
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 10s PAUSE																
ONE_WIRE_WAIT_10us:
	nop
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 100s PAUSE																
ONE_WIRE_WAIT_100us:
	push temp1          ; temp1 auf dem Stack sichern            
;------------
	ldi  temp1,(10*TAKT/1000000)
ONE_WIRE_WAIT_100us_w: 
    WDR					; WATCH-DOG		
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	dec  temp1
    brne ONE_WIRE_WAIT_100us_w
;------------
	pop temp1      		; temp1 wiederherstellen                    
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 500s PAUSE																
ONE_WIRE_WAIT_500us:
	push temp1          ; temp1 auf dem Stack sichern            
;------------
	ldi  temp1,125
wait_500us_w: 
    WDR					; WATCH-DOG		
	dec  temp1
    brne wait_500us_w
;------------
	pop temp1      		; temp1 wiederherstellen                    
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ca. 800ms PAUSE																
ONE_WIRE_WAIT_800ms:
	push temp1          ; temp1 auf dem Stack sichern            
	push temp2          ; temp2 auf dem Stack sichern            
	push temp3          ; temp3 auf dem Stack sichern            

;------------
	clr	temp1			
	clr temp2
	ldi	temp3,3		
ONE_WIRE_WAIT_800ms_w:	
    WDR					; WATCH-DOG		
	dec	temp1
	brne ONE_WIRE_WAIT_800ms_w
	dec	temp2		
	brne ONE_WIRE_WAIT_800ms_w
	dec	temp3
	brne ONE_WIRE_WAIT_800ms_w
;------------
	pop temp3      		; temp3 wiederherstellen                    
	pop temp2      		; temp2 wiederherstellen                    
	pop temp1      		; temp1 wiederherstellen                    
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
