; ##############################################################################
; ##############################################################################
; ##############################################################################
; x33 senden und 8 BYTES empfangen und im SRAM ablegen							
DS_ROM_READ:
	; RESET										
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX x33 (read ROM)					
	ldi	temp1,0x33			
	rcall ONE_WIRE_DATA_WRITE
	; MASTER RX 8 BYTE und im SRAM ablegen		
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+0),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+1),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+2),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+3),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+4),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+5),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+6),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_ROM_BEREICH+7),temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; xBE senden und 9 BYTES empfangen und im SRAM ablegen							
DS_SCRATCHPAD_READ:
	; RESET										
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX xCC (SKIP ROM)					
	ldi	temp1,0xCC
	rcall ONE_WIRE_DATA_WRITE
	; MASTER TX xBE (read SCRATCHPAD)					
	ldi	temp1,0xBE			
	rcall ONE_WIRE_DATA_WRITE
	; MASTER RX 9 BYTE und im SRAM ablegen		
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+0),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+1),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+2),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+3),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+4),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+5),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+6),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+7),temp1
	rcall ONE_WIRE_DATA_READ
	STS(adr_DS_SCRATCHPAD_BEREICH+8),temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; x4E senden  (2 BYTES senden und im SCRATCHPAD ablegen)						
DS_SCRATCHPAD_WRITE:
	; RESET										
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX xCC (SKIP ROM)					
	ldi	temp1,0xCC
	rcall ONE_WIRE_DATA_WRITE
	; MASTER TX x4E (WRITE SCRATCHPAD)					
	ldi	temp1,0x4E			
	rcall ONE_WIRE_DATA_WRITE
	; DATENBYTE (high)
	ldi	temp1,0x22
	rcall ONE_WIRE_DATA_WRITE
	; DATENBYTE (LOW)
	ldi	temp1,0x11
	rcall ONE_WIRE_DATA_WRITE

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; x48 senden (COPPY SCRATCHPAD in EEPROM)										
DS_SCRATCHPAD_COPPY:
	; RESET										
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX xCC (SKIP ROM)					
	ldi	temp1,0xCC
	rcall ONE_WIRE_DATA_WRITE
	; MASTER TX x48 (COPPY SCRATCHPAD in EEPROM)
	ldi	temp1,0x48			
	rcall ONE_WIRE_DATA_WRITE
	; voll POWER (strong pullup )				
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	; Pause (mind. 10ms)						
	rcall ONE_WIRE_WAIT_800ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; xB8 senden (COPPY EEPROM in SCRATCHPAD)										
DS_SCRATCHPAD_RECALLEEPROM:
	; RESET										
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX xCC (SKIP ROM)					
	ldi	temp1,0xCC
	rcall ONE_WIRE_DATA_WRITE
	; MASTER TX xB8 (COPPY EEPROM in SCRATCHPAD)
	ldi	temp1,0xB8			
	rcall ONE_WIRE_DATA_WRITE
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
DS_MESSEN_ALLE:
	;Reset und Presence-Test					
	rcall ONE_WIRE_RESET_PRESENCE
	; MASTER TX xCC (skip ROM)					
	ldi	temp1,0xCC			
	rcall ONE_WIRE_DATA_WRITE	
	; MASTER TX x44 (convert T)					
	ldi	temp1,0x44
	rcall ONE_WIRE_DATA_WRITE	
	; voll POWER (strong pullup )				
	sbi (ONE_WIRE_PORT),(ONE_WIRE_PORTBIT)	; HIGH-PEGEL 	
	sbi (ONE_WIRE_DDR), (ONE_WIRE_PORTBIT)	; AUF AUSGANG	
	; Pause 									
	cbi PORTD,LED_D_GELB	; LED GELB an
	rcall ONE_WIRE_WAIT_800ms
	sbi PORTD,LED_D_GELB	; LED GELB aus
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Berechnung der Temperatur mit 0,5 GRD GENAUIGKEIT								
; inp: temp1+2																	
; OUT: temp1+2																	
DS_TEMPERATUR_BERECHNEN_9BIT:
	; TESTZAHLEN	
	;ldi temp1, 0xCE	; - 25
	;ldi temp2, 0xFF

	; negativ ?
	tst temp2
	brmi DS_TEMPERATUR_BERECHNEN_9BIT_NEGATV
	rjmp DS_TEMPERATUR_BERECHNEN_9BIT_POSITIV
;-------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_9BIT_POSITIV:
	; BERECHNUNG	
	rcall DS_TEMPERATUR_BERECHNEN_9BIT_B
	ret
;-------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_9BIT_NEGATV:
	; invertieren +1
	com temp1
	com temp2
	add temp1,EINS
	adc temp2,NULL
	; BERECHNUNG	
	rcall DS_TEMPERATUR_BERECHNEN_9BIT_B
	; invertieren +1
	com temp1
	com temp2
	add temp1,EINS
	adc temp2,NULL
	ret
;-------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_9BIT_B:
	; mal 5 (4 mal addieren)			
	mov temp,temp1

	add temp1,temp	; x2
	adc temp2,NULL
	add temp1,temp	; x3
	adc temp2,NULL
	add temp1,temp	; x4
	adc temp2,NULL
	add temp1,temp	; x5
	adc temp2,NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Berechnung der Temperatur mit 0,1 GRD GENAUIGKEIT								
;																				
; TEMPERATUR= TEMP_READ - 0,25 + (COUNT_PER_C - COUNT_REMAIN)/COUNT_PER_C		
;																				
; TEMPERATUR= TEMP_READ - 0,25 + (16 - COUNT_REMAIN)/16							
;																				
; TEMPERATUR= TEMP_READ - 2 + (160 - 10*COUNT_REMAIN)/16						
;																				
; inp: temp1+2+6(COUNT_REMAIN)													
DS_TEMPERATUR_BERECHNEN:
	; TESTZAHLEN	
	;ldi temp1,0xCE	; -25
	;ldi temp2,0xFF
	;ldi temp6,8

	; negativ ?
	tst temp2
	brmi DS_TEMPERATUR_BERECHNEN_NEGATIV 
	rjmp DS_TEMPERATUR_BERECHNEN_POSITIV
;----------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_POSITIV:
	; BIT 0 clear (0,5 GRD ignorieren)	
	andi temp1,0b11111110
	; mal 5 (4 mal addieren)			
	; ERGEBNIS: Bsp. 210 fr 21 GRD		
	mov temp,temp1

	add temp1,temp	; x2
	adc temp2,NULL
	add temp1,temp	; x3
	adc temp2,NULL
	add temp1,temp	; x4
	adc temp2,NULL
	add temp1,temp	; x5
	adc temp2,NULL
	; BERECHNEN			
	rcall DS_TEMPERATUR_BERECHNEN_B

	; addieren			
	add temp1,temp6	; 
	adc temp2,NULL	
	; -0,25 (-2)		
	subi temp1,LOW (2)
	sbci temp2,HIGH(2)
	ret
;----------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_NEGATIV:
	; invertieren	
	com temp1
	com temp2
	add temp1,EINS
	adc temp2,NULL
	; BIT 0 clear (0,5 GRD ignorieren)	
	andi temp1,0b11111110
	; mal 5 (4 mal addieren)			
	; ERGEBNIS: Bsp. 210 fr 21 GRD		
	mov temp,temp1

	add temp1,temp	; x2
	adc temp2,NULL
	add temp1,temp	; x3
	adc temp2,NULL
	add temp1,temp	; x4
	adc temp2,NULL
	add temp1,temp	; x5
	adc temp2,NULL



	; BERECHNEN			
	rcall DS_TEMPERATUR_BERECHNEN_B
	; 9-temp6			
	ldi temp,9
	sub temp,temp6
	mov temp6,temp

	cpi temp6,0
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W0
	cpi temp6,1
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W1
	cpi temp6,2
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W2
	cpi temp6,3
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W3
	cpi temp6,4
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W4
	cpi temp6,5
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W5
	cpi temp6,6
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W6
	cpi temp6,7
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W7
	cpi temp6,8
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W8
	cpi temp6,9
	breq DS_TEMPERATUR_BERECHNEN_NEGATIV_W9


DS_TEMPERATUR_BERECHNEN_NEGATIV_W0:
	subi temp1,(-3)	;+3
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W1:
	subi temp1,(-4)	;+4
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W2:
	subi temp1,(-5)	;+5
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W3:
	subi temp1,(-6)	;+6
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W4:
	subi temp1,(-7)	;+7
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W5:
	subi temp1,(2)	;-2
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W6:
	subi temp1,(1)	;-1
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W7:
	subi temp1,(0)	;0
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W8:
	subi temp1,(-1)	;+1
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG
DS_TEMPERATUR_BERECHNEN_NEGATIV_W9:
	subi temp1,(-2)	;+2
	rjmp DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG





DS_TEMPERATUR_BERECHNEN_NEGATIV_FERTIG:
	; invertieren	
	com temp1
	com temp2
	add temp1,EINS
	adc temp2,NULL
	ret
;----------------------------------------------------------------------------------
DS_TEMPERATUR_BERECHNEN_B:
	; COUNT_REMAIN (temp6) mal 10 		
	mov temp,temp6
	LSL temp6		; x2 (x2) 
	LSL	temp6		; x2 (x4) 
	LSL temp6		; x2 (x8) 
	add temp6,temp	; +  (x9) 
	add temp6,temp	; +	 (x10)
	; 160- temp6	
	ldi temp,160
	sub temp, temp6
	mov temp6,temp
	; temp6 aufrunden (halben DIVISOR addieren)
	subi temp6,(-8)
	; temp6 durch 16		
	LSR temp6		; :2 (:2)
	LSR temp6		; :2 (:4)
	LSR temp6		; :2 (:8)
	LSR temp6		; :2 (:16)

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
